//
//  JsonHelper.cpp
//  NDSEngineB2D
//
//  Created by Justin Hawkwood on 2013/08/06.
//
//

#include "JsonHelper.h"

#include <string>
#include <iostream>
#include <vector>

#include "json/stringbuffer.h"
#include "json/writer.h"
#include "json/prettywriter.h"

#define loc_debug true

using namespace std;
using namespace cocos2d;

Object* objectFromJson(rapidjson::Value::ConstValueIterator valueIt)
{
    if (valueIt == NULL)
        return NULL;
    
    if (valueIt->IsObject())
    {
        Dictionary *dictionary = Dictionary::create();
        
		rapidjson::Value::ConstMemberIterator itr = valueIt->MemberonBegin();
        while (itr != valueIt->MemberonEnd()) {
			const char *key = itr->name.GetString();
            dictionary->setObject(objectFromJson(&itr->value), string(key));
			
			itr++;
		}
        
        return dictionary;
    }
    else if (valueIt->IsArray())
    {
		int size = valueIt->Size();
        Array *array = Array::createWithCapacity(size);
		
		for (rapidjson::Value::ConstValueIterator itr = valueIt->onBegin(); itr != valueIt->onEnd(); itr++) {
			array->addObject(objectFromJson(itr));
		}
        
        return array;
    }
	// else convert anything else to a string
	else if (valueIt->IsBool())
    {
        bool ret = valueIt->GetBool();
		return String::create( (ret ? "1" : "0") );
    }
    else if (valueIt->IsInt())
    {
        int val = 0;
		val = valueIt->GetInt();
        
        String *String = String::createWithFormat( "%i", val);
        return String;
    }
    else if (valueIt->IsUint())
    {
        uint val = 0;
		val = valueIt->GetUint();
        
        String *String = String::createWithFormat( "%ui", val);
        return String;
    }
    else if (valueIt->IsInt64())
    {
        long long val = 0;
		val = valueIt->GetInt();
        
        String *String = String::createWithFormat( "%lli", val);
        return String;
    }
    else if (valueIt->IsUint64())
    {
        unsigned long long val = 0;
		val = valueIt->GetUint64();
        
        String *String = String::createWithFormat( "%llu", val);
        return String;
    }
    else if (valueIt->IsDouble())
    {
        double val = 0;
		val = valueIt->GetDouble();
        
        String *String = String::createWithFormat( "%lf", val);
        return String;
    }
    else if (valueIt->IsNumber())
    {
		double val = 0.0;
		val = valueIt->GetDouble();
        
        String *String = String::createWithFormat( "%lf", val);
        return String;
    }
    else if (valueIt->IsString())
    {
        String *String = String::create( valueIt->GetString() );
        return String;
    }
    
    return NULL;
}

rapidjson::Value* jsonFromObject(Object* obj, rapidjson::Document *document)
{
	bool isAllocatorDocument = false;
	if (!document)
	{
		document = new rapidjson::Document();
		isAllocatorDocument = true;
	}
	
	rapidjson::Value *retVal = nullptr;
    
	if (dynamic_cast<Dictionary*>(obj))
    {
        Dictionary *mainDict = (Dictionary*)obj;
        Array *allKeys = mainDict->allKeys();
        
		retVal = new rapidjson::Value(rapidjson::kObjectType);
        
        if (allKeys != NULL ) {
			for (unsigned int i = 0; i < allKeys->count(); i++)
			{
				const char *key = ((String*)allKeys->objectAtIndex(i))->getCString();
				rapidjson::Value *value2 = jsonFromObject(mainDict->objectForKey(key), document);
				retVal->AddMember( key, value2, document->GetAllocator() );
				delete value2;
			}
		}
    }
    else if (dynamic_cast<Array*>(obj))
    {
        Array* mainArray = (Array*)obj;
		retVal = new rapidjson::Value(rapidjson::kArrayType);
		
        for (unsigned int i = 0; i < mainArray->count(); i++)
        {
			rapidjson::Value *value2 = jsonFromObject(mainArray->objectAtIndex(i), document);
			retVal->PushBack( &value2, document->GetAllocator() );
			delete value2;
        }
    }
    else // if (dynamic_cast<String*>(obj)) // treat everything else as string
    {
        String* str = (String*)obj;
		const char *str2 = str->getCString();
		retVal = new rapidjson::Value( str2 );
    }
    
	if (isAllocatorDocument)
	{
		delete document;
	}
    return retVal;
}

bool saveObjectToJSONFile(Object* obj, const char* fullPath)
{
	if ((obj == NULL) || (fullPath == NULL)) {
#if loc_debug
		log("JsonHelper: cannot saveObjectToFile() with NULL arguments");
#endif
		return false;
	}
	
	string dump = getJSONStringFromObject(obj, true);
	
	FILE *fp = fopen(fullPath, "w");
	if (!fp) {
#if loc_debug
		log("JsonHelper: FAILED TO OPEN FILE TO %s", fullPath);
#endif
		return false;
	}
	int rit = fwrite(dump.c_str(), dump.size(), 1, fp);
	fclose(fp);
	
	if (rit == 0) {
#if loc_debug
		log("JsonHelper: FAILED TO WRITE FILE TO %s", fullPath);
#endif
		return false;
	}
	
#if loc_debug
	log("JsonHelper: SUCCESSFULLY WROTE FILE TO %s", fullPath);
#endif
	
	return true;
}

Object *getObjectFromJSONFile(const char * filepath)
{
	if (filepath == NULL) {
#if loc_debug
		log("JsonHelper: cannot loadObjectFromFile() with NULL arguments");
#endif
		return NULL;
	}
	
	std::string contentStr = FileUtils::getInstance()->getStringFromFile(filepath);
	
	if (contentStr.length() < 2) {
#if loc_debug
		log("JsonHelper: no data at %s", filepath);
#endif
		return NULL;
	}
	
	Object* obj = getObjectFromJSONString(contentStr);
	CCAssert(obj != NULL, String::createWithFormat("... in file %s", filepath)->getCString());
	return obj;
}

Object *getObjectFromJSONString(String *str)
{
	if (!str || (str->length() == 0)) {
		log("WARNING: getObjectFromJSONString: str empty");
		return NULL;
	}
	
	std::string jsonStr = std::string(str->getCString());
	return getObjectFromJSONString(jsonStr);
}

Object *getObjectFromJSONString(const char *str)
{
	if (!str || (strlen(str) == 0)) {
		log("WARNING: getObjectFromJSONString: str empty");
		return NULL;
	}

	std::string jsonStr = std::string(str);
	return getObjectFromJSONString(jsonStr);
}

Object *getObjectFromJSONString(const string& str)
{
	rapidjson::Document doc;
	
	doc.Parse<0>(str.c_str());
	if( doc.HasParseError())
	{
        log("GetParseError %s\n", doc.GetParseError());
		return NULL;
	}
	
    Object *retVal = objectFromJson(&doc);
	
	return retVal;
}


string getJSONStringFromObject(Object *obj)
{
	return getJSONStringFromObject(obj, false);
}

string getJSONStringFromObject(Object *obj, bool pretty)
{
    rapidjson::Value *value = jsonFromObject(obj, NULL);
    rapidjson::StringBuffer buffer;
    if (pretty) {
		rapidjson::PrettyWriter<rapidjson::StringBuffer> writer(buffer);
		value->Accept(writer);
    }
	else {
		rapidjson::Writer<rapidjson::StringBuffer> writer(buffer);
		value->Accept(writer);
	}
	std::string jsonString = buffer.GetString();
	delete value;
    return jsonString;
}


